#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>
#include <gdk/gdk.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include <time.h>

#include "callbacks.h"
#include "interface.h"
#include "support.h"


/* Some local variable */
static GtkCList *list;         /* copy of CList */
static gchar file_cmpts[256];  /* fichier de comptes */
static int nbops=0, currop=0;  /* nombre d'operations et operation courrante */

static guint year, month, day;           /* date */
static gchar *description, *payby;       /* nature et mode de paiement */
static gchar *amount, *money_name;       /* montant de l'operation et unite monetaire */

#define CREDIT 0
#define DEBIT 1
static gint operation;                   /* credit/debit */

static gfloat solde_alert = 0.00;  /* seuil d'alerte de solde trop bas */


/* Correspondance numero/mois */
static gchar *get_month(guint m) {
  switch(m) { 
  case 1: return "Jan";
  case 2: return "Feb";
  case 3: return "Mar";
  case 4: return "Apr";
  case 5: return "May";
  case 6: return "Jun";
  case 7: return "Jul";
  case 8: return "Aug";
  case 9: return "Sep";
  case 10: return "Oct";
  case 11: return "Nov";
  case 12: return "Dec";
  default: return "???"; 
  }
}
/* Ajoute un zero en tete du nombre */
static gchar *zero_add(guint n) {
  if(n<10) return g_strdup_printf("0%d", n);
  else return g_strdup_printf("%d", n);
}
/* Renvoie la date formatee (a partir de aaaa/mm/dd) */
static gchar *get_date(gchar *d) {
  gchar *date=(gchar*)malloc(sizeof(gchar)*strlen(d));
  gchar *aaaa, *mm, *dd;

  strcpy(date, d);
  aaaa = strtok(date, "/\n"); mm = strtok(NULL, "/\n"); dd = strtok(NULL, "/\n");
  mm = get_month(atoi(mm)); /* conversion numero->mois */
  date = g_strdup_printf("%s %s %s", dd, mm, aaaa);
  
  return date;
}
/* Renvoie le montant du credit ou du debit */
static gchar *get_credit(gchar *amount, gint operation) {
  gchar *credit=(gchar*)malloc(sizeof(gchar)*strlen(amount));

  if(operation==CREDIT)      
    return credit = g_strdup_printf("%s", g_strdup_printf("%.2f", atof(amount)));
  return credit = g_strdup(" ");
}
static gchar *get_debit(gchar *amount, gint operation) {
  gchar *debit=(gchar*)malloc(sizeof(gchar)*strlen(amount));

  if(operation==DEBIT)      
    return debit = g_strdup_printf("%s", g_strdup_printf("%.2f", atof(amount)));
  return debit = g_strdup(" ");
}

/* Read CList (file.cmpts) */
/* Date|Desc|Payby|Credit|Debit|Tot */
static void read_cmpts_clist(GtkCList *clist, gchar *filename) {
  FILE *file;
  char line[1024], *fin;
  char *tks="|\n";
  char *str;
  gchar *txt[7];

  /* Ouverture du fichier cmpts */
  if(!(file=fopen(filename, "rb"))) {
    fprintf(stderr, "Can't open cmpts file(Cmpts) : %s\n", filename);
    exit(-2);
  }
  fprintf(stdout, "Loading cpt file : %s", filename);

  nbops = currop = 0;
  gtk_clist_clear(clist);
  gtk_clist_freeze(clist);

  /* Lecture des operations */
  while((fin=fgets(line, 1024, file))!=NULL) {
    int j;

    /* Nombre d'operations */
    nbops = currop+1;

    /* Lecture des champs */
    txt[0] = g_strdup_printf("%s", str = strtok(line, tks));
    txt[2] = g_strdup_printf("%s", strtok(NULL, tks));
    txt[3] = g_strdup_printf("%s", strtok(NULL, tks));

    str = strtok(NULL, tks); /* montant du credit/debit */
    if(str!=NULL) {
      if(atof(str)>=0) {
	txt[4] = g_strdup_printf("%s", str);
	txt[5] = g_strdup(" ");
      }
      else if(atof(str)<0) {
	txt[4] = g_strdup(" ");
	txt[5] = g_strdup_printf("%s", str+1);
      }
    }
    else {
      txt[4] = g_strdup(" ");
      txt[5] = g_strdup(" ");
    }

    txt[6] = g_strdup_printf("%s", strtok(NULL, tks));
    txt[1] = get_date(txt[0]); /* formatage de la date */
    /* Ajout de l'operation */
    gtk_clist_append(clist, txt);
    gtk_clist_set_row_data(clist, currop, g_strdup_printf("%s|%s|%s|%s|%s|%s",
							  txt[0], txt[2], txt[3], 
							  txt[4], txt[5], txt[6]));
    for(j=0; j<7; j++)
      g_free(txt[j]);

    currop++;
  }
  currop--; /* on est alle trop loin */

  gtk_clist_thaw(clist);

  /* Fermeture du fichier cpt */
  fclose(file);
  fprintf(stdout, "\n");

  /* Selection de la derniere operation */
  gtk_clist_select_row(clist, currop, 0);
}

/* Save CList (file.cmpts2) */
static void save_cmpts2_clist(GtkCList *clist, gchar *filename) {
  FILE *file;
  int i;
  gchar *txt;

  /* Ouverture du fichier cmpts2 */
  if(!(file=fopen(filename, "w"))) {
    fprintf(stderr, "Can't open cmpts2 file(Cmpts2) : %s\n", filename);
    exit(-2);
  }
  fprintf(stdout, "Writing cmpts2 file : %s", filename);

  for(i=0; i<nbops; i++) {
    /*fprintf(file, "%s\n", (char*)gtk_clist_get_row_data(clist, i));*/
    gtk_clist_get_text(clist, i, 0, &txt);
    fprintf(file, "%s|", (char*)txt);
    gtk_clist_get_text(clist, i, 2, &txt);
    fprintf(file, "%s|", (char*)txt);
    gtk_clist_get_text(clist, i, 3, &txt);
    fprintf(file, "%s|", (char*)txt);
    gtk_clist_get_text(clist, i, 4, &txt); 
    if(strcmp(txt, " ")) fprintf(file, "%s|", (char*)txt);
    gtk_clist_get_text(clist, i, 5, &txt); 
    if(strcmp(txt, " ")) fprintf(file, "-%s|", (char*)txt);
    gtk_clist_get_text(clist, i, 6, &txt);
    fprintf(file, "%s\n", (char*)txt);
  }
  
  /* Fermeture du fichier cmpts2 */
  fclose(file);
  fprintf(stdout, "\n");
}

/* Save CList (file.html) */
static void save_html_clist(GtkCList *clist, gchar *filename) {
  FILE *file;
  int i;
  gchar *txt;

  /* Ouverture du fichier html */
  if(!(file=fopen(filename, "w"))) {
    fprintf(stderr, "Can't open html file(html) : %s\n", filename);
    exit(-2);
  }
  fprintf(stdout, "Writing html file : %s", filename);

  fprintf(file, "<!doctype html public \"-//w3c//dtd html 4.0 transitional//en\">\n");
  fprintf(file, "<html>\n<head>\n");
  fprintf(file, "<meta http-equiv=\"Content-Type\" content=\"text/html; charset=iso-8859-1\">\n");
  fprintf(file, "<meta name=\"GENERATOR\" content=\"Mozilla/4.7 [en] (X11; I; Linux 2.2.14-15mdk i686) [Netscape]\"></head>\n");
  fprintf(file, "<body text=\"#000000\" bgcolor=\"#F1F4E9\">\n");

  fprintf(file, "<center><b><font size=+3>Gcpt - Accounts Book</font></b>\n");
  fprintf(file, "<p><b><font size=+3>Mikael 'Mickey' NAVARRO</font></b></center>\n");

  fprintf(file, "<br>&nbsp;\n");
  fprintf(file, "<p><b><font size=+1>%s</font></b>\n", filename);

  fprintf(file, "<center><table BORDER WIDTH=\"100%%\" BGCOLOR=\"#FFFFFF\" NOSAVE >\n");
  fprintf(file, "<tr BGCOLOR=\"#DAFFE9\" NOSAVE>\n");
  fprintf(file, "<td><center><b><font size=+1>Date</font></b></center></td>\n");
  fprintf(file, "<td><b><font size=+1>Description</font></b></td>\n");
  fprintf(file, "<td><center><b><font size=+1>Payment</font></b></center></td>\n");
  fprintf(file, "<td><div align=right><b><font size=+1>Credit</font></b></div></td>\n");
  fprintf(file, "<td><div align=right><b><font size=+1>Debit</font></b></div></td>\n");
  fprintf(file, "<td><div align=right><b><font size=+1>Total</font></b></div></td>\n</tr>");

  for(i=0; i<nbops; i++) {
    /*fprintf(file, "%s\n", (char*)gtk_clist_get_row_data(clist, i));*/
    fprintf(file, "<tr BGCOLOR=\"#F1F4E9\" NOSAVE>\n");
    gtk_clist_get_text(clist, i, 1, &txt);
    fprintf(file, "<td><center>%s</center></td>\n", (char*)txt);
    gtk_clist_get_text(clist, i, 2, &txt);
    fprintf(file, "<td>%s</td>\n", (char*)txt);
    gtk_clist_get_text(clist, i, 3, &txt);
    fprintf(file, "<td><center>%s</center></td>\n", (char*)txt);
    gtk_clist_get_text(clist, i, 4, &txt); 
    fprintf(file, "<td><div align=right>%s</div></td>\n", (char*)txt);
    gtk_clist_get_text(clist, i, 5, &txt); 
    fprintf(file, "<td><div align=right>%s</div></td>\n", (char*)txt);
    gtk_clist_get_text(clist, i, 6, &txt);
    fprintf(file, "<td><div align=right>%s</div></td>\n", (char*)txt);
    fprintf(file, "</tr>\n");
  }

  fprintf(file, "</table></center>\n");
  fprintf(file, "</body>\n");
  fprintf(file, "</html>\n");

  /* Fermeture du fichier html */
  fclose(file);
  fprintf(stdout, "\n");
}

/* Save CList (file.tex) */
static void save_latex_clist(GtkCList *clist, gchar *filename) {
  FILE *file;
  int i;
  gchar *txt;

  /* Ouverture du fichier LaTeX */
  if(!(file=fopen(filename, "w"))) {
    fprintf(stderr, "Can't open LaTeX file(tex) : %s\n", filename);
    exit(-2);
  }
  fprintf(stdout, "Writing LaTeX file : %s", filename);

  fprintf(file, "\\documentclass[a4paper,11pt]{article}\n\n");
  fprintf(file, "\\usepackage[latin1]{inputenc}\n");
  fprintf(file, "\\usepackage[T1]{fontenc}\n");
  fprintf(file, "\\usepackage[francais]{babel}\n\n");

  fprintf(file, "\\pagestyle{empty}\n\n");
  fprintf(file, "\\title{Gcpt - Accounts Book}\n");
  fprintf(file, "\\author{ Mikael 'mickey' Navarro}\n\n");

  fprintf(file, "\\begin{document}\n\n");

  fprintf(file, "\\parindent = 0pt\n");
  fprintf(file, "\\parskip = 0pt\n");
  fprintf(file, "\\voffset = -3cm\n");
  fprintf(file, "\\vsize = 27cm\n");
  fprintf(file, "\\hoffset = -3cm\n");
  fprintf(file, "\\hsize = 15cm\n\n");

  fprintf(file, "\\maketitle\n\n");
  fprintf(file, "\\begin\n{em}%s\n\\end{em}\n\n", filename);

  fprintf(file, "\\begin{tabular}{||c|p{7cm}|c|r|r|r||}\n");
  fprintf(file, "\\hline\n");
  fprintf(file, "Date & Description & Payment & Credit & Debit & Total \\\\ \n");
  fprintf(file, "\\hline\n");
  fprintf(file, "\\hline\n");

  for(i=0; i<nbops; i++) {
    /*fprintf(file, "%s\n", (char*)gtk_clist_get_row_data(clist, i));*/
    gtk_clist_get_text(clist, i, 1, &txt);
    fprintf(file, "%s &", (char*)txt);
    gtk_clist_get_text(clist, i, 2, &txt);
    fprintf(file, "%s &", (char*)txt);
    gtk_clist_get_text(clist, i, 3, &txt);
    fprintf(file, "%s &", (char*)txt);
    gtk_clist_get_text(clist, i, 4, &txt); 
    fprintf(file, "%s &", (char*)txt);
    gtk_clist_get_text(clist, i, 5, &txt); 
    fprintf(file, "%s &", (char*)txt);
    gtk_clist_get_text(clist, i, 6, &txt);
    fprintf(file, "%s \\\\ \n", (char*)txt);
    if(i!=0 && i%50==0) {  
      fprintf(file, "\\hline\n");
      fprintf(file, "\\end{tabular}\n");

      fprintf(file, "\\begin{tabular}{||c|p{7cm}|c|r|r|r||}\n");
      fprintf(file, "\\hline\n");
      fprintf(file, "Date & Description & Payment & Credit & Debit & Total \\\\ \n");
      fprintf(file, "\\hline\n");
      fprintf(file, "\\hline\n");
    }
  }
  
  fprintf(file, "\\hline\n");
  fprintf(file, "\\end{tabular}\n");
  fprintf(file, "\\end{document}\n\n");

  /* Fermeture du fichier LaTeX */
  fclose(file);
  fprintf(stdout, "\n");
}

/* Update CList (totaux) */
static void update_clist(GtkCList *clist) {
  int i;
  gchar *credit, *debit, *cheque;
  double t=0.0, c=0.0, d=0.0;
  GdkColor col_red, col_black;
  GdkColor col_green, col_white;

  col_red.red = 65535; col_red.green = 0; col_red.blue = 0;
  col_black.red = 0; col_black.green = 0; col_black.blue = 0;
  col_green.red = 60000; col_green.green = 65535; col_green.blue = 60000;
  col_white.red = 65535; col_white.green = 65535; col_white.blue = 65535;

  /* Mise a jour des totaux */
  for(i=0; i<nbops; i++) {
    gtk_clist_get_text(clist, i, 4, &credit); /* credit */
    if(credit) c = atof(credit); else c = 0.0;
    gtk_clist_get_text(clist, i, 5, &debit); /* debit */
    if(debit) d = atof(debit); else d = 0.0;
    t = t + c - d; /* update totaux */
    gtk_clist_set_text(clist, i, 6, g_strdup_printf("%.2f", t));

    /* Colorisation */
    if(t<=solde_alert) gtk_clist_set_foreground(clist, i, &col_red);
    else     gtk_clist_set_foreground(clist, i, &col_black);
    if(i%2) gtk_clist_set_background(clist, i, &col_green);
    else    gtk_clist_set_background(clist, i, &col_white);

    /* recup dernier num de cheque */
    gtk_clist_get_text(clist, i, 3, &cheque);
    if(strcmp(cheque, "CARD") && strcmp(cheque, "BKTRANSF"))
      payby = g_strdup(cheque);
  } 
}


/* window : Init CList */
void on_clist_realize(GtkWidget *widget, gpointer user_data) {
  /* Justification des colonnes */
  gtk_clist_set_column_justification(GTK_CLIST(user_data), 0, GTK_JUSTIFY_LEFT);
  gtk_clist_set_column_justification(GTK_CLIST(user_data), 1, GTK_JUSTIFY_CENTER);
  gtk_clist_set_column_justification(GTK_CLIST(user_data), 2, GTK_JUSTIFY_LEFT);
  gtk_clist_set_column_justification(GTK_CLIST(user_data), 3, GTK_JUSTIFY_CENTER);
  gtk_clist_set_column_justification(GTK_CLIST(user_data), 4, GTK_JUSTIFY_RIGHT);
  gtk_clist_set_column_justification(GTK_CLIST(user_data), 5, GTK_JUSTIFY_RIGHT);
  gtk_clist_set_column_justification(GTK_CLIST(user_data), 6, GTK_JUSTIFY_RIGHT);
  /* On cache la colonne des SDate, utilise pour le tri */
  gtk_clist_set_column_visibility(GTK_CLIST(user_data), 0, FALSE);
  /* Parametrage du tri */
  gtk_clist_set_sort_column(GTK_CLIST(user_data), 0);
  gtk_clist_set_sort_type(GTK_CLIST(user_data), GTK_SORT_ASCENDING);
  gtk_clist_set_auto_sort(GTK_CLIST(user_data), TRUE);
  /* Parametrage des lignes */
  gtk_clist_set_reorderable(GTK_CLIST(user_data), FALSE);

  /* Copy of CList */
  list = GTK_CLIST(user_data);
  /* Some inits */
  money_name = g_strdup("Euro");
  operation = CREDIT;
  amount = g_strdup("0.00");
  payby = g_strdup("0");
}

/* window : Selection d'une ligne */
void on_clist_select_row(GtkCList *clist, gint row, gint column, GdkEvent *event, gpointer user_data) {
  currop = row;
}

/* window : Menu sur bouton droit et edition sur double click */
gboolean on_clist_button_press_event(GtkWidget *widget, GdkEventButton *event, gpointer user_data) {
  /* Menu contextuel */
  if(event->button == 3 && GTK_CLIST(widget)->selection) {
    GtkWidget *menu;

    menu = create_menu();
    gtk_menu_popup(GTK_MENU(menu), NULL, NULL, NULL, NULL, event->button, event->time);
  }

  /* dbClick -> edit */
  if(event->type == GDK_2BUTTON_PRESS) {
    if(nbops>0) {
      GtkWidget *edit;
    
      edit = create_edit();
      gtk_widget_show(edit);
    }
  }

  return FALSE;
}

/* menu : Menu contextuel */
void on_new_activate(GtkMenuItem *menuitem, gpointer user_data) {
  GtkWidget *new;

  new = create_new();
  gtk_widget_show(new);
}
void on_edit_activate(GtkMenuItem *menuitem, gpointer user_data) {
  if(nbops>0) {
    GtkWidget *edit;
    
    edit = create_edit();
    gtk_widget_show(edit);
  }
}
void on_erase_activate(GtkMenuItem *menuitem, gpointer user_data) {
  if(nbops>0) {
    /* On efface la ligne selectionnee */
    gtk_clist_remove(list, currop);
    nbops--;

    /* On selectionne la ligne precedente, si elle existe */
    if(nbops>0) {
      if(currop==0)
	gtk_clist_select_row(list, 0, 0);
      else if(currop>0) {
	currop--;
	gtk_clist_select_row(list, currop, 0);
      }

      /* Update totaux */
      update_clist(list);
    }
  }
}
void on_params_activate(GtkMenuItem *menuitem, gpointer user_data) {
  GtkWidget *options;
  
  options = create_options();
  gtk_widget_show(options);
}


/* params : Solde minlevel alert */
void on_spinbutton_alert_realize(GtkWidget *widget, gpointer user_data) {
  gtk_spin_button_set_value(GTK_SPIN_BUTTON(user_data), solde_alert);
}
void on_spinbutton_alert_changed(GtkEditable *editable, gpointer user_data) {
  solde_alert = gtk_spin_button_get_value_as_float(GTK_SPIN_BUTTON(user_data));
  update_clist(list);
}

/* params : View/hide Description, Payment column */
void on_togglebutton_desc_realize(GtkWidget *widget, gpointer user_data) {
}
void on_togglebutton_desc_toggled(GtkToggleButton *togglebutton, gpointer user_data) {
  if(GTK_TOGGLE_BUTTON(user_data)->active) {
    gtk_clist_set_column_visibility(list, 2, TRUE);
  } 
  else {
    gtk_clist_set_column_visibility(list, 2, FALSE);
  }
}
void on_togglebutton_payment_realize(GtkWidget *widget, gpointer user_data) {
}
void on_togglebutton_payment_toggled(GtkToggleButton *togglebutton, gpointer user_data) {
  if(GTK_TOGGLE_BUTTON(user_data)->active) {
    gtk_clist_set_column_visibility(list, 3, TRUE);
  } 
  else {
    gtk_clist_set_column_visibility(list, 3, FALSE);
  }
}


/* window : About Gcpt2 ? */
void on_about_button_clicked(GtkButton *button, gpointer user_data) {
  system("netscape /usr/local/doc/gcpt/gcpt.html &");
}

/* window : Load new file_cmpts */
void on_load_button_clicked(GtkButton *button, gpointer user_data) {
  GtkWidget *filew;

  filew = create_win_load();
  gtk_widget_show(filew);
}
/* win_load : Accept new file_cmpts */
void on_loadfile_button_clicked(GtkButton *button, gpointer user_data) {
  strcpy(file_cmpts, gtk_file_selection_get_filename(GTK_FILE_SELECTION(user_data)));
  gtk_widget_destroy(GTK_WIDGET(user_data));

  /* Read cpt file and update totaux */
  read_cmpts_clist(list, file_cmpts);
  update_clist(list);
}


/* window : Save current account */
void on_saveas_button_clicked(GtkButton *button, gpointer user_data) {
  GtkWidget *filew;

  filew = create_win_save();
  gtk_widget_show(filew);
}
/* win_save : Okey save as ... */
void on_save_button_clicked(GtkButton *button, gpointer user_data) {
  gchar file[256], tfile[256];

  strcpy(file, gtk_file_selection_get_filename(GTK_FILE_SELECTION(user_data)));
  gtk_widget_destroy(GTK_WIDGET(user_data));

  if(strrchr(file, '.')) {
    strcpy(tfile, strrchr(file, '.'));
    if(!strcmp(tfile, ".cmpts2"))
      save_cmpts2_clist(list, file); /* Save cmpts file (ver2) */
    else if(!strcmp(tfile, ".html"))
      save_html_clist(list, file); /* Save html file */
    else if(!strcmp(tfile, ".tex"))
      save_latex_clist(list, file); /* Save LaTeX file */
    else {
      strcpy(file, strcat(file, ".cmpts2"));
      save_cmpts2_clist(list, file);
    }
  }
  else {
    strcpy(file, strcat(file, ".cmpts2"));
    save_cmpts2_clist(list, file);
  }
}


/* window : Erase current op */
void on_erase_button_clicked(GtkButton *button, gpointer user_data) {
  if(nbops>0) {
    /* On efface la ligne selectionnee */
    gtk_clist_remove(GTK_CLIST(user_data), currop);
    nbops--;

    /* On selectionne la ligne precedente, si elle existe */
    if(nbops>0) {
      if(currop==0)
	gtk_clist_select_row(GTK_CLIST(user_data), 0, 0);
      else if(currop>0) {
	currop--;
	gtk_clist_select_row(GTK_CLIST(user_data), currop, 0);
      }

      /* Update totaux */
      update_clist(GTK_CLIST(user_data));
    }
  }
}


/* window : AddNew operation */
void on_new_button_clicked(GtkButton *button, gpointer user_data) {
  GtkWidget *new;

  new = create_new();
  gtk_widget_show(new);
}
/* new : Accept new operation */
void on_accept_button_clicked(GtkButton *button, gpointer user_data) {
  gchar *txt[7];
  int j;

  /* Destruction de la fenetre de saisie -> fin (pb mem) */
  /*gtk_widget_destroy(GTK_WIDGET(user_data));*/

  /* Recuperation des champs */
  txt[0] = g_strdup_printf("%s/%s/%s", 
		    g_strdup_printf("%d", year), 
		    zero_add(month), 
		    zero_add(day));
  txt[1] = get_date(txt[0]); /* formatage de la date */
  txt[2] = g_strdup_printf("%s", (strlen(description))? description : " ");
  txt[3] = g_strdup_printf("%s", (strlen(payby))? payby : " ");
  txt[4] = get_credit(amount, operation);
  txt[5] = get_debit(amount, operation);
  txt[6] = g_strdup(" ");

  /* Ajout de la ligne dans la clist */
  currop = gtk_clist_append(list, txt);
  gtk_clist_set_row_data(list, currop, g_strdup_printf("%s|%s|%s|%s|%s|%s",
						       txt[0], txt[2], txt[3], 
						       txt[4], txt[5], txt[6]));
  for(j=0; j<7; j++)
    g_free(txt[j]);
  nbops++;

  /* Selection de l'operation ajoute */
  gtk_clist_select_row(list, currop, 0);

  /* Update totaux */
  update_clist(list);

  /* Destruction de la fenetre de saisie */
  gtk_widget_destroy(GTK_WIDGET(user_data));
}

/* new : Inits */
void on_new_realize(GtkWidget *widget, gpointer user_data) {
}

/* new : Mise a jour de la date */
void on_calendar_realize(GtkWidget *widget, gpointer user_data) {
  struct tm *date;
  time_t t = time(NULL);

  date = localtime(&t);
  gtk_calendar_select_day(GTK_CALENDAR(user_data), date->tm_mday);

  gtk_calendar_get_date(GTK_CALENDAR(user_data), &year, &month, &day);
  month++; /* ca commence a 0 !? */
}
void on_calendar_day_selected(GtkCalendar *calendar, gpointer user_data) {
  gtk_calendar_get_date(GTK_CALENDAR(user_data), &year, &month, &day);
  month++; /* ca commence a 0 !? */
}
void on_calendar_month_changed(GtkCalendar *calendar, gpointer user_data) {
  gtk_calendar_get_date(GTK_CALENDAR(user_data), &year, &month, &day);
  month++; /* ca commence a 0 !? */
}
void on_calendar_next_year(GtkCalendar *calendar, gpointer user_data) {
  gtk_calendar_get_date(GTK_CALENDAR(user_data), &year, &month, &day);
  month++; /* ca commence a 0 !? */
}
void on_calendar_prev_year(GtkCalendar *calendar, gpointer user_data) {
  gtk_calendar_get_date(GTK_CALENDAR(user_data), &year, &month, &day);
  month++; /* ca commence a 0 !? */
}

/* new : Mise a jour du montant de l'operation */
void on_entry_amount_realize(GtkWidget *widget, gpointer user_data) {
  gtk_entry_set_text(GTK_ENTRY(user_data), "0.00");
}
void on_entry_amount_changed(GtkEditable *editable, gpointer user_data) {
  amount = gtk_entry_get_text(GTK_ENTRY(user_data));
}

/* new : Mise a jour de la nature de l'operation */
void on_entry_desc_realize(GtkWidget *widget, gpointer user_data) {
  gtk_entry_set_text(GTK_ENTRY(user_data), "");  
}
void on_entry_desc_changed(GtkEditable *editable, gpointer user_data) {
  description = gtk_entry_get_text(GTK_ENTRY(user_data));
}

/* new : Nature de l'operation */
void on_radiobutton_credit_realize(GtkWidget *widget, gpointer user_data) {
  operation = CREDIT;
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(user_data), TRUE);
}
void on_radiobutton_debit_realize(GtkWidget *widget, gpointer user_data) {
}
void on_radiobutton_credit_toggled(GtkToggleButton *togglebutton, gpointer user_data) {
  operation = CREDIT;
}
void on_radiobutton_debit_toggled(GtkToggleButton *togglebutton, gpointer user_data) {
  operation = DEBIT;
}

/* new : Unite monetaire */
void on_radiobutton_euro_toggled(GtkToggleButton *togglebutton, gpointer user_data) {
  money_name = g_strdup("Euro");
}
void on_radiobutton_franc_toggled(GtkToggleButton *togglebutton, gpointer user_data) {
  money_name = g_strdup("Franc");
}
void on_radiobutton_other_toggled(GtkToggleButton *togglebutton, gpointer user_data) {
  money_name = gtk_entry_get_text(GTK_ENTRY(user_data));
}
void on_entry_other_changed(GtkEditable *editable, gpointer user_data) {
  money_name = gtk_entry_get_text(GTK_ENTRY(user_data));
}

/* new : Mode de payment */
void on_radiobutton_cheque_realize(GtkWidget *widget, gpointer user_data) {
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(user_data), TRUE);    
}
void on_spinbutton_cheque_realize(GtkWidget *widget, gpointer user_data) {
  assert(payby);
  if(strcmp(payby, "CARD") && strcmp(payby, "BKTRANSF"))
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(user_data), atof(payby));
  else
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(user_data), 0);
}
void on_radiobutton_card_realize(GtkWidget *widget, gpointer user_data) {
}
void on_radiobutton_transfer_realize(GtkWidget *widget, gpointer user_data) {
}
void on_radiobutton_cheque_toggled(GtkToggleButton *togglebutton, gpointer user_data) {
  payby = g_strdup_printf("%d", 
			  gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(user_data)));
}
void on_spinbutton_cheque_changed(GtkEditable *editable, gpointer user_data) {
  payby = g_strdup_printf("%d", 
			  gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(user_data)));
}
void on_radiobutton_card_toggled(GtkToggleButton *togglebutton, gpointer user_data) {
  payby = g_strdup("CARD");
}
void on_radiobutton_transfer_toggled(GtkToggleButton *togglebutton, gpointer user_data) {
  payby = g_strdup("BKTRANSF");
}


/* window : Edit current operation */
void on_edit_button_clicked(GtkButton *button, gpointer user_data) {
  if(nbops>0) {
    GtkWidget *edit;
    
    edit = create_edit();
    gtk_widget_show(edit);
  }
}
/* edit : Accept modifications */
void on_change_button_clicked(GtkButton *button, gpointer user_data) {
  gchar *txt[7];
  int j;

  /* Destruction de la fenetre de saisie -> fin (pb mem) */
  /*gtk_widget_destroy(GTK_WIDGET(user_data));*/

  /* On efface l'ancienne operation (ligne) */
  gtk_clist_remove(list, currop);

  /* Recuperation des champs */
  txt[0] = g_strdup_printf("%s/%s/%s", 
		    g_strdup_printf("%d", year), 
		    zero_add(month), 
		    zero_add(day));
  txt[1] = get_date(txt[0]); /* formatage de la date */
  txt[2] = g_strdup_printf("%s", (strlen(description))? description : " ");
  txt[3] = g_strdup_printf("%s", (strlen(payby))? payby : " ");
  txt[4] = get_credit(amount, operation);
  txt[5] = get_debit(amount, operation);
  txt[6] = g_strdup(" ");

  /* Ajout de la ligne dans la clist */
  currop = gtk_clist_append(list, txt);
  gtk_clist_set_row_data(list, currop, g_strdup_printf("%s|%s|%s|%s|%s|%s",
						       txt[0], txt[2], txt[3], 
						       txt[4], txt[5], txt[6]));
  for(j=0; j<7; j++)
    g_free(txt[j]);

  /* Selection de l'operation modifiee */
  gtk_clist_select_row(list, currop, 0);

  /* Update totaux */
  update_clist(list);

  /* Destruction de la fenetre de saisie */
  gtk_widget_destroy(GTK_WIDGET(user_data));
}

/* edit : Inits */
void on_edit_realize(GtkWidget *widget, gpointer user_data) {
  /* Recup des donnees */
  if(currop>=0) {
    gchar *date, *amount_c, *amount_d;
    gchar *tdate;
    
    /* Date */
    gtk_clist_get_text(list, currop, 0, &date);
    tdate = g_strdup(date);
    year  = atoi(strtok(tdate, "/\n"));
    month = atoi(strtok(NULL, "/\n"));
    day   = atoi(strtok(NULL, "/\n"));;

    /* Description et moyen de paiement */
    gtk_clist_get_text(list, currop, 2, &description);
    gtk_clist_get_text(list, currop, 3, &payby);

    /* Montant et type de l'operation */
    gtk_clist_get_text(list, currop, 4, &amount_c);
    gtk_clist_get_text(list, currop, 5, &amount_d);
    if(strcmp(amount_c, " ")) { operation=CREDIT; amount = amount_c; }
    else if(strcmp(amount_d, " ")) { operation=DEBIT; amount = amount_d; }
  }
}

/* edit : Mise a jour de la date - modification */
void on_calendar_edit_realize(GtkWidget *widget, gpointer user_data) {
  guint yyyy, mm, dd;
  
  assert(year>=0);
  assert(1<=month && month<=12);
  assert(1<=day && day<=31);
  yyyy = year; mm = month; dd = day;

  gtk_calendar_select_month(GTK_CALENDAR(user_data), mm-1, yyyy);
  gtk_calendar_select_day(GTK_CALENDAR(user_data), dd);
}
void on_calendar_edit_day_selected(GtkCalendar *calendar, gpointer user_data) {
  gtk_calendar_get_date(GTK_CALENDAR(user_data), &year, &month, &day);
  month++; /* ca commence a 0 !? */
}
void on_calendar_edit_month_changed(GtkCalendar *calendar, gpointer user_data) {
  gtk_calendar_get_date(GTK_CALENDAR(user_data), &year, &month, &day);
  month++; /* ca commence a 0 !? */
}
void on_calendar_edit_next_year(GtkCalendar *calendar, gpointer user_data) {
  gtk_calendar_get_date(GTK_CALENDAR(user_data), &year, &month, &day);
  month++; /* ca commence a 0 !? */
}
void on_calendar_edit_prev_year(GtkCalendar *calendar, gpointer user_data) {
  gtk_calendar_get_date(GTK_CALENDAR(user_data), &year, &month, &day);
  month++; /* ca commence a 0 !? */
}

/* edit : Modification du montant de l'operation */
void on_entry_edit_amount_realize(GtkWidget *widget, gpointer user_data) {
  assert(amount);
  gtk_entry_set_text(GTK_ENTRY(user_data), amount);
}
void on_entry_edit_amount_changed(GtkEditable *editable, gpointer user_data) {
  amount = gtk_entry_get_text(GTK_ENTRY(user_data));
}

/* edit : Modification de la nature de l'operation */
void on_entry_edit_desc_realize(GtkWidget *widget, gpointer user_data) {
  assert(description);
  if(!strcmp(description, " ")) gtk_entry_set_text(GTK_ENTRY(user_data), "");
  else gtk_entry_set_text(GTK_ENTRY(user_data), description);
}
void on_entry_edit_desc_changed(GtkEditable *editable, gpointer user_data) {
  description = gtk_entry_get_text(GTK_ENTRY(user_data));
}

/* edit : Nature de l'operation */
void on_radiobutton_edit_credit_realize(GtkWidget *widget, gpointer user_data) {
  if(operation==CREDIT) gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(user_data), TRUE);
}
void on_radiobutton_edit_debit_realize(GtkWidget *widget, gpointer user_data) {
  if(operation==DEBIT) gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(user_data), TRUE);
}
void on_radiobutton_edit_credit_toggled(GtkToggleButton *togglebutton, gpointer user_data) {
  operation = CREDIT;
}
void on_radiobutton_edit_debit_toggled(GtkToggleButton *togglebutton, gpointer user_data) {
  operation = DEBIT;
}

/* edit : Modification de la monaie */
void on_radiobutton_edit_euro_toggled(GtkToggleButton *togglebutton, gpointer user_data) {
  money_name = g_strdup("Euro");
}
void on_radiobutton_edit_franc_toggled(GtkToggleButton *togglebutton, gpointer user_data) {
  money_name = g_strdup("Franc");
}
void on_radiobutton_edit_other_toggled(GtkToggleButton *togglebutton, gpointer user_data) {
  money_name = gtk_entry_get_text(GTK_ENTRY(user_data));
}
void on_entry_edit_other_changed(GtkEditable *editable, gpointer user_data) {
  money_name = gtk_entry_get_text(GTK_ENTRY(user_data));
}

/* edit : Modificatin du mode de payment */
void on_radiobutton_edit_cheque_realize(GtkWidget *widget, gpointer user_data) {
  assert(payby);
  if(strcmp(payby, "CARD") && strcmp(payby, "BKTRANSF"))
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(user_data), TRUE);    
}
void on_spinbutton_edit_cheque_realize(GtkWidget *widget, gpointer user_data) {
  assert(payby);
  if(strcmp(payby, "CARD") && strcmp(payby, "BKTRANSF"))
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(user_data), atof(payby));
}
void on_radiobutton_edit_card_realize(GtkWidget *widget, gpointer user_data) {
  assert(payby);
  if(!strcmp(payby, "CARD"))
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(user_data), TRUE);    
}
void on_radiobutton_edit_transfer_realize(GtkWidget *widget, gpointer user_data) {
  assert(payby);
  if(!strcmp(payby, "BKTRANSF"))
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(user_data), TRUE);    
}
void on_radiobutton_edit_cheque_toggled(GtkToggleButton *togglebutton, gpointer user_data) {
  payby = g_strdup_printf("%d", 
			  gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(user_data)));
}
void on_spinbutton_edit_cheque_changed(GtkEditable *editable, gpointer user_data) {
  payby = g_strdup_printf("%d", 
			  gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(user_data)));
}
void on_radiobutton_edit_card_toggled(GtkToggleButton *togglebutton, gpointer user_data) {
  payby = g_strdup("CARD");
}
void on_radiobutton_edit_transfer_toggled(GtkToggleButton *togglebutton, gpointer user_data) {
  payby = g_strdup("BKTRANSF");
}


/* window : View balance */
void on_view_button_clicked(GtkButton *button, gpointer user_data) {
  if(nbops>0) {
    GtkWidget *balance;
    
    balance = create_balance();
    gtk_widget_show(balance);
  }
}

/* balance : Construction de la liste epargnes/benefices */
void on_clist_balance_realize(GtkWidget *widget, gpointer user_data) {
  gchar *aaaa, *mm, *currmm;
  gchar *date, *tdate, *cred, *deb;
  gfloat c=0.0, d=0.0;
  gchar *txt[4];
  int i, j, l;
  GdkColor col_red, col_black;
  GdkColor col_green, col_white;

  col_red.red = 65535; col_red.green = 0; col_red.blue = 0;
  col_black.red = 0; col_black.green = 0; col_black.blue = 0;
  col_green.red = 60000; col_green.green = 65535; col_green.blue = 60000;
  col_white.red = 65535; col_white.green = 65535; col_white.blue = 65535;

  /* Justification des colonnes */
  gtk_clist_set_column_justification(GTK_CLIST(user_data), 0, GTK_JUSTIFY_CENTER);
  gtk_clist_set_column_justification(GTK_CLIST(user_data), 1, GTK_JUSTIFY_RIGHT);
  gtk_clist_set_column_justification(GTK_CLIST(user_data), 2, GTK_JUSTIFY_RIGHT);
  gtk_clist_set_column_justification(GTK_CLIST(user_data), 3, GTK_JUSTIFY_RIGHT);
  /* On gele la liste */
  gtk_clist_clear(GTK_CLIST(user_data));
  gtk_clist_freeze(GTK_CLIST(user_data));

  /* Initialisation de l'iteration */
  gtk_clist_get_text(list, 0, 0, &date);
  tdate = (gchar*)malloc(sizeof(gchar)*strlen(date));
  strcpy(tdate, date);
  aaaa = strtok(tdate, "/\n"); currmm = strtok(NULL, "/\n");
  gtk_clist_get_text(list, 0, 4, &cred);
  gtk_clist_get_text(list, 0, 5, &deb);
  c += atof(cred);
  d += atof(deb);

  /* Parcours de la liste des operations */
  for(i=1; i<nbops; i++) {      
    gtk_clist_get_text(list, i, 0, &date);
    tdate = (gchar*)malloc(sizeof(gchar)*strlen(date));
    strcpy(tdate, date);
    aaaa = strtok(tdate, "/\n"); mm = strtok(NULL, "/\n");

    if(atoi(mm) == atoi(currmm)) {
      /* Mise a jour du credit/debit total */
      gtk_clist_get_text(list, i, 4, &cred);
      gtk_clist_get_text(list, i, 5, &deb);
      c += atof(cred);
      d += atof(deb);
    }
    else {
      /* Formatage de la date : currmm/aaaa */
      txt[0] = g_strdup_printf("%s %s", get_month(atoi(currmm)), aaaa);
      /* Calcul de la balance */
      txt[1] = g_strdup_printf("%.2f", c);
      txt[2] = g_strdup_printf("%.2f", d);
      txt[3] = g_strdup_printf("%.2f", c - d);

      /* Ajout de l'operation */
      l = gtk_clist_append(GTK_CLIST(user_data), txt);
      /* Colorisation */
      if(c-d<=0.0) gtk_clist_set_foreground(GTK_CLIST(user_data), l, &col_red);
      else     gtk_clist_set_foreground(GTK_CLIST(user_data), l, &col_black);
      if(l%2) gtk_clist_set_background(GTK_CLIST(user_data), l, &col_green);
      else    gtk_clist_set_background(GTK_CLIST(user_data), l, &col_white);

      /* Remise a zero */
      c = 0.0;
      d = 0.0;
      /* Mise a jour du credit/debit total du nouveau mois */
      gtk_clist_get_text(list, i, 4, &cred);
      gtk_clist_get_text(list, i, 5, &deb);
      c += atof(cred);
      d += atof(deb);

      /* Mise a jour du mois courrant */
      currmm = g_strdup_printf("%s", mm);

      /* Libaration memoire */
      for(j=0; j<4; j++)
	g_free(txt[j]);
    }
  }
  
  /* Affichage du dernier mois */
  txt[0] = g_strdup_printf("%s %s", get_month(atoi(currmm)), aaaa);
  /* Calcul de la balance */
  txt[1] = g_strdup_printf("%.2f", c);
  txt[2] = g_strdup_printf("%.2f", d);
  txt[3] = g_strdup_printf("%.2f", c - d);
  /* Ajout de l'operation */
  l = gtk_clist_append(GTK_CLIST(user_data), txt);
  /* Colorisation */
  if(c-d<=0.0) gtk_clist_set_foreground(GTK_CLIST(user_data), l, &col_red);
  else     gtk_clist_set_foreground(GTK_CLIST(user_data), l, &col_black);
  if(l%2) gtk_clist_set_background(GTK_CLIST(user_data), l, &col_green);
  else    gtk_clist_set_background(GTK_CLIST(user_data), l, &col_white);

  /* Libaration memoire */
  for(j=0; j<4; j++)
    g_free(txt[j]);

  /* Degele */
  gtk_clist_thaw(GTK_CLIST(user_data));
}

/* Affichage du solde crediteur */
void on_entry_solde_realize(GtkWidget *widget, gpointer user_data) {
  gchar *solde;
  
  gtk_clist_get_text(list, nbops-1, 6, &solde);
  gtk_entry_set_text(GTK_ENTRY(user_data), solde);
}




